Weak

Struct Weak 

pub struct Weak<T: ?Sized> { /* private fields */ }
Available on crate feature weak only.
A weak reference to a ThinCell that doesn’t prevent dropping.

Weak references don’t keep the value alive. The value will be dropped when all strong references (ThinCell) are dropped, even if weak references still exist.

Use Weak::upgrade to attempt to convert a weak reference back to a strong reference (ThinCell). This will fail if the value has already been dropped.

Implementations

Source

impl<T: ?Sized> Weak<T>

pub fn upgrade(&self) -> Option<ThinCell<T>>

Attempts to upgrade the weak reference to a strong reference.

Returns Some(ThinCell) if the value still exists, or None if it has been dropped.

Examples
let cell = ThinCell::new(42);
let weak = cell.downgrade();

let strong = weak.upgrade().unwrap();
assert_eq!(*strong.borrow(), 42);

drop(cell);
drop(strong);
assert!(weak.upgrade().is_none());
pub fn strong_count(&self) -> usize

Returns the number of strong references.

pub fn weak_count(&self) -> usize

Returns the number of weak references.

pub fn as_ptr(&self) -> *const ()

Gets a raw pointer to the inner allocation.

The pointer is valid only if the strong count is non-zero.